import { evaluateParsedConditions, getOverwritenDefaults, mountElement, parseConditionString, unmountElement, } from './utils';
import { BUTTON_CLASS_DEFAULT, BUTTON_CONTAINER_CLASS_DEFAULT, BUTTON_ELEMENT, CLICK_ATTRIBUTE, CONTAINER_DEFINITION, DIV_ELEMENT, ID_ATTRIBUTE, TYPE_ATTRIBUTE, } from './constants';
export class Button {
    options = {
        id: '',
        type: 'button',
        buttonType: 'submit',
        className: BUTTON_CLASS_DEFAULT,
        template: 'Submit',
    };
    containerElement = null;
    buttonElement = null;
    _id;
    _parent;
    _form;
    _isMounted = false;
    _isVisible = true;
    _type;
    _parsedConditions = null;
    constructor(parent, form, options) {
        this.initializeOptions(options);
        this._parent = parent;
        this._form = form;
        this._id = this.options.id;
        this._type = options.type;
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, getOverwritenDefaults(this.options.type, options));
    }
    async initialize() {
        this.parseStringConditions();
        await this.reset();
    }
    parseStringConditions() {
        if (typeof this.options.conditions === 'string') {
            this._parsedConditions = parseConditionString(this.options.conditions);
        }
    }
    getId() {
        return this._id;
    }
    getContainer() {
        return this.containerElement;
    }
    getType() {
        return this._type;
    }
    getButtonType() {
        return this.options.buttonType;
    }
    getVisibility() {
        return this._isVisible;
    }
    createContainerElement() {
        this.containerElement = document.createElement(DIV_ELEMENT);
        this.containerElement.className = BUTTON_CONTAINER_CLASS_DEFAULT;
        this.containerElement.setAttribute(ID_ATTRIBUTE, this._id + CONTAINER_DEFINITION);
    }
    createButtonElement() {
        this.buttonElement = document.createElement(BUTTON_ELEMENT);
        this.buttonElement.className = this.options.className;
        this.buttonElement.setAttribute(ID_ATTRIBUTE, this._id);
        this.buttonElement.setAttribute(TYPE_ATTRIBUTE, this.options.buttonType);
        this.buttonElement.innerHTML = this.options.template;
        this.buttonElement.addEventListener(CLICK_ATTRIBUTE, (event) => this.onClick(event));
    }
    onGui() {
        this.createContainerElement();
        this.createButtonElement();
        if (this.containerElement && this.buttonElement)
            mountElement(this.buttonElement, this.containerElement);
    }
    mount() {
        if (this.buttonElement)
            mountElement(this.buttonElement, this._parent);
        this._isMounted = true;
    }
    unmount() {
        if (this.buttonElement)
            unmountElement(this.buttonElement);
        this._isMounted = false;
    }
    destroy() {
        if (this._parent)
            this._parent.remove();
    }
    handleVisibility() {
        if (this._isVisible && !this._isMounted)
            this.mount();
        if (!this._isVisible && this._isMounted)
            this.unmount();
    }
    onClick(event) {
        if (this.options.click)
            this.options.click(event, this._form.getData());
    }
    async reset() {
        this.update();
    }
    async update() {
        this.updateVisibilityBasedOnConditions();
        this.handleVisibility();
    }
    updateVisibilityBasedOnConditions() {
        if (this.options.conditions) {
            if (this._parsedConditions) {
                this._isVisible = evaluateParsedConditions(this._parsedConditions, this._form.getData());
            }
            else if (typeof this.options.conditions === 'function') {
                this._isVisible = this.options.conditions(this._form.getData());
            }
        }
    }
}
