"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CheckboxField = void 0;
const constants_1 = require("../constants");
const field_1 = require("../field");
const utils_1 = require("../utils");
class CheckboxField extends field_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: constants_1.FIELD_TYPE_CHECKBOX,
            required: false,
            validation: (value, data, required) => {
                if (required && !value)
                    return constants_1.DEFAULT_REQUIRED_VALIDATION_MESSAGE;
                return true;
            },
            default: null,
            className: constants_1.CHECKBOX_CLASS_DEFAULT,
            toggle: false,
        };
        this.sliderElement = null;
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    syncValue() {
        if (this.inputElement && this.inputElement instanceof HTMLInputElement) {
            if (this.getValue() && !this.inputElement.hasAttribute(constants_1.CHECKED_ATTRIBUTE)) {
                this.inputElement.setAttribute(constants_1.CHECKED_ATTRIBUTE, String(this.getValue()));
            }
            else if (!this.getValue() && this.inputElement.hasAttribute(constants_1.CHECKED_ATTRIBUTE))
                this.inputElement.removeAttribute(constants_1.CHECKED_ATTRIBUTE);
        }
    }
    createContainerElement() {
        this.containerElement = document.createElement(constants_1.DIV_ELEMENT);
        this.containerElement.className = constants_1.FIELD_CLASS_DEFAULT + ' ' + this.getType();
        if (this.options.toggle)
            this.containerElement.className += ' ' + constants_1.TOGGLE_CLASS_DEFAULT;
        this.containerElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId() + constants_1.CONTAINER_DEFINITION);
    }
    createLabelElement() {
        this.labelElement = document.createElement(constants_1.LABEL_ELEMENT);
        if (this.inputElement && this.labelElement)
            (0, utils_1.mountElement)(this.inputElement, this.labelElement);
        if (this.sliderElement && this.labelElement)
            (0, utils_1.mountElement)(this.sliderElement, this.labelElement);
        const label = document.createElement(constants_1.PARAGRAPH_ELEMENT);
        if (this.options.label)
            label.innerText = this.options.label;
        label.setAttribute(constants_1.ID_ATTRIBUTE, this.getId() + constants_1.LABEL_DEFINITION);
        this.labelElement.setAttribute(constants_1.FOR_ATTRIBUTE, this.options.name || this.getId());
        label.className = constants_1.LABEL_CLASS_DEFAULT;
        if (this.options.label)
            (0, utils_1.mountElement)(label, this.labelElement);
    }
    createInputElement() {
        this.inputElement = document.createElement(constants_1.INPUT_ELEMENT);
        this.inputElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId());
        this.inputElement.setAttribute(constants_1.NAME_ATTRIBUTE, this.options.name || this.getId());
        this.inputElement.setAttribute(constants_1.TYPE_ATTRIBUTE, this.getType());
        if (this.options.default)
            this.inputElement.setAttribute(constants_1.CHECKED_ATTRIBUTE, String(this.options.default));
        this.inputElement.className = this.options.className;
    }
    createToggleSpan() {
        this.sliderElement = document.createElement(constants_1.SPAN_ELEMENT);
        this.sliderElement.className = constants_1.TOGGLE_SLIDER_CLASS_DEFAULT;
    }
    onGui() {
        this.createContainerElement();
        if (this.options.toggle)
            this.createToggleSpan();
        this.createInputElement();
        this.createLabelElement();
        this.createValidationElement();
        if (this.containerElement && this.labelElement)
            (0, utils_1.mountElement)(this.labelElement, this.containerElement);
        if (this.labelElement && this.validationElement)
            (0, utils_1.mountElement)(this.validationElement, this.labelElement);
    }
    bindChange() {
        if (this.inputElement)
            this.inputElement.addEventListener(constants_1.CHANGE_ATTRIBUTE, (event) => {
                this.change(event);
            });
    }
    change(event) {
        this.setValue(event.target.checked);
        if (this.options.change)
            this.options.change(this.getValue());
        this.validate();
    }
    getValue() {
        return this._value;
    }
}
exports.CheckboxField = CheckboxField;
