"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileField = void 0;
const filepond_1 = require("filepond");
const field_1 = require("../field");
const utils_1 = require("../utils");
const constants_1 = require("../constants");
class FileField extends field_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: constants_1.FIELD_TYPE_FILE,
            required: false,
            validation: (value, data, required) => {
                if (required && !value)
                    return constants_1.DEFAULT_REQUIRED_VALIDATION_MESSAGE;
                return true;
            },
            className: constants_1.INPUT_CLASS_DEFAULT,
            enhance: true,
            multiple: false,
            debounce: 50,
            options: {
                storeAsFile: true,
                instantUpload: false,
                allowProcess: false,
            },
        };
        this._filepond = null;
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
            this.initFilepond();
            this.bindChange();
        });
    }
    initializeOptions(options) {
        this.options = Object.assign({}, Object.assign(Object.assign({}, this.options), {
            options: { allowMultiple: options.multiple },
        }), (0, utils_1.getOverwritenDefaults)(this.options.type, options));
    }
    syncValue() {
    }
    createInputElement() {
        this.inputElement = document.createElement(constants_1.INPUT_ELEMENT);
        this.inputElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId());
        this.inputElement.setAttribute(constants_1.NAME_ATTRIBUTE, this.options.name || this.getId());
        this.inputElement.setAttribute(constants_1.TYPE_ATTRIBUTE, this.getType());
        if (this.options.placeholder)
            this.inputElement.setAttribute(constants_1.PLACEHOLDER_ATTRIBUTE, this.options.placeholder);
        this.inputElement.className = this.options.className;
        if (this.options.multiple && !this.options.enhance)
            this.inputElement.setAttribute(constants_1.MULTIPLE_ATTRIBUTE, 'true');
        if (this.options.accept)
            this.inputElement.setAttribute(constants_1.ACCEPT_ATTRIBUTE, this.options.accept);
    }
    getFilepond() {
        return this._filepond;
    }
    initFilepond() {
        if (this.inputElement && this.options.enhance) {
            this._filepond = (0, filepond_1.create)(this.inputElement, this.options.options || {});
        }
    }
    bindChange() {
        var _a;
        if (this.options.enhance) {
            if (!this._filepond)
                return;
            this._filepond.on('addfile', (error, file) => {
                if (error)
                    return;
                const files = this._filepond.getFiles();
                this.filePondChange(files);
            });
            this._filepond.on('removefile', (error, file) => {
                if (error)
                    return;
                const files = this._filepond.getFiles();
                this.filePondChange(files);
            });
            this._filepond.on('updatefiles', (error, file) => {
                if (error)
                    return;
                const files = this._filepond.getFiles();
                this.filePondChange(files);
            });
        }
        else {
            (_a = this.inputElement) === null || _a === void 0 ? void 0 : _a.addEventListener(constants_1.CHANGE_ATTRIBUTE, (0, utils_1.debounce)(this.change, this.options.debounce, this));
        }
    }
    filePondChange(files) {
        this.changeValue(files);
    }
    change(event) {
        var _a;
        if ((_a = event.target) === null || _a === void 0 ? void 0 : _a.files) {
            this.changeValue(event.target.files);
        }
    }
    changeValue(value) {
        this.setValue(value);
        this.validate();
        this.handleValidatedField();
        if (this.options.change)
            this.options.change(this._value);
    }
    getValue() {
        return this._value;
    }
}
exports.FileField = FileField;
