"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HiddenField = void 0;
const utils_1 = require("../utils");
const constants_1 = require("../constants");
class HiddenField {
    constructor(parent, form, options) {
        this.options = {
            id: '',
            type: constants_1.FIELD_TYPE_HIDDEN,
            default: null,
        };
        this.inputElement = null;
        this.containerElement = null;
        this.labelElement = null;
        this.validationElement = null;
        this._value = null;
        this.initializeOptions(options);
        this._id = this.options.id;
        this._form = form;
        this._parent = parent;
        this._type = options.type;
        this._saveKey = (0, utils_1.generateFieldSaveKey)(this._form.getId(), this._id);
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, (0, utils_1.getOverwritenDefaults)(this.options.type, options));
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
        });
    }
    setValue(value, save = true) {
        this._value = value;
        this.syncValue();
        this._form.setData(this._id, value);
        if (save)
            this.save();
    }
    getValue() {
        return this._value;
    }
    syncValue() {
        if (this.inputElement && this.inputElement instanceof HTMLInputElement && this.inputElement.value !== this._value) {
            this.inputElement.value = this._value;
        }
    }
    mountToParent() {
        if (this.containerElement)
            (0, utils_1.mountElement)(this.containerElement, this._parent);
    }
    createContainerElement() {
        this.containerElement = document.createElement(constants_1.DIV_ELEMENT);
        this.containerElement.className = constants_1.FIELD_CLASS_DEFAULT + ' ' + this.getType();
        this.containerElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId() + constants_1.CONTAINER_DEFINITION);
    }
    createInputElement() {
        this.inputElement = document.createElement(constants_1.INPUT_ELEMENT);
        this.inputElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId());
        this.inputElement.setAttribute(constants_1.NAME_ATTRIBUTE, this.options.name || this.getId());
        this.inputElement.setAttribute(constants_1.TYPE_ATTRIBUTE, this.getType());
    }
    getId() {
        return this._id;
    }
    getSaveKey() {
        return this._saveKey;
    }
    getType() {
        return this._type;
    }
    getForm() {
        return this._form;
    }
    onGui() {
        this.createContainerElement();
        this.createInputElement();
        if (this.containerElement && this.inputElement)
            (0, utils_1.mountElement)(this.inputElement, this.containerElement);
        this.mountToParent();
    }
    destroy() {
        if (this._parent)
            this._parent.remove();
    }
    save() {
        if (this._form.savesProgress() && this._form.hasValidLicense() && this._value !== undefined) {
            localStorage.setItem(this._saveKey, JSON.stringify(this._value));
        }
    }
    load() {
        var _a;
        if (this._form.savesProgress() && this._form.hasValidLicense()) {
            const value = localStorage.getItem(this._saveKey);
            if (value !== null) {
                this.setValue((0, utils_1.isJson)(value) ? JSON.parse(value) : value, false);
                return;
            }
        }
        this.setValue((_a = this.options.default) !== null && _a !== void 0 ? _a : null, false);
    }
    reset() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            localStorage.removeItem(this._saveKey);
            this.setValue((_a = this.options.default) !== null && _a !== void 0 ? _a : null, false);
            this.update();
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            return;
        });
    }
}
exports.HiddenField = HiddenField;
