"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NumberField = void 0;
const constants_1 = require("../constants");
const field_1 = require("../field");
class NumberField extends field_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: constants_1.FIELD_TYPE_NUMBER,
            required: false,
            validation: (value, data, required) => {
                if (required && !value && value !== 0)
                    return constants_1.DEFAULT_REQUIRED_VALIDATION_MESSAGE;
                if (this.options.min && this.options.min > value)
                    return constants_1.BIGGER_THAN_VALIDATION_MESSAGE + this.options.min;
                if (this.options.max && this.options.max < value)
                    return constants_1.LESS_THAN_VALIDATION_MESSAGE + this.options.max;
                return true;
            },
            default: null,
            className: constants_1.INPUT_CLASS_DEFAULT,
        };
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
        this.bindMinMax();
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
            this.bindMinMax();
            this.bindChange();
        });
    }
    bindMinMax() {
        if (this.inputElement) {
            if (this.options.min)
                this.inputElement.setAttribute(constants_1.MIN_ATTRIBUTE, String(this.options.min));
            if (this.options.max)
                this.inputElement.setAttribute(constants_1.MAX_ATTRIBUTE, String(this.options.max));
            if (this.options.step)
                this.inputElement.setAttribute(constants_1.STEP_ATTRIBUTE, String(this.options.step));
        }
    }
    getValue() {
        return this._value;
    }
}
exports.NumberField = NumberField;
