"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RadioField = void 0;
const constants_1 = require("../constants");
const field_1 = require("../field");
const utils_1 = require("../utils");
class RadioField extends field_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: constants_1.FIELD_TYPE_RADIO,
            schema: [],
            required: false,
            validation: (value, data, required) => {
                if (required && !value)
                    return constants_1.DEFAULT_REQUIRED_VALIDATION_MESSAGE;
                return true;
            },
            default: null,
            className: constants_1.RADIO_CLASS_DEFAULT,
        };
        this.inputElements = [];
        this.labelElements = [];
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
            this.handleVisibility();
        });
    }
    syncValue() {
        this.inputElements.forEach((el) => {
            if (el.value !== this.getValue() && el.hasAttribute(constants_1.CHECKED_ATTRIBUTE))
                el.removeAttribute(constants_1.CHECKED_ATTRIBUTE);
            else if (el.value === this.getValue() && !el.hasAttribute(constants_1.CHECKED_ATTRIBUTE))
                el.setAttribute(constants_1.CHECKED_ATTRIBUTE, '');
        });
    }
    createRadioLabelElement(labelText, id, input, container) {
        const labelElement = document.createElement(constants_1.LABEL_ELEMENT);
        if (input && labelElement)
            (0, utils_1.mountElement)(input, labelElement);
        const label = document.createElement(constants_1.PARAGRAPH_ELEMENT);
        if (labelText)
            label.innerText = labelText;
        label.setAttribute(constants_1.ID_ATTRIBUTE, id + constants_1.LABEL_DEFINITION);
        labelElement.setAttribute(constants_1.FOR_ATTRIBUTE, id);
        label.className = constants_1.LABEL_CLASS_DEFAULT;
        labelElement.append(label);
        this.labelElements.push(labelElement);
        if (container && labelElement)
            (0, utils_1.mountElement)(labelElement, container);
    }
    createRadioInputElement(id, value) {
        const inputElement = document.createElement(constants_1.INPUT_ELEMENT);
        inputElement.setAttribute(constants_1.ID_ATTRIBUTE, id);
        inputElement.setAttribute(constants_1.NAME_ATTRIBUTE, this.getId());
        inputElement.setAttribute(constants_1.VALUE_ATTRIBUTE, value);
        if (this.options.default && value === this.options.default)
            inputElement.setAttribute(constants_1.CHECKED_ATTRIBUTE, 'true');
        inputElement.setAttribute(constants_1.TYPE_ATTRIBUTE, this.getType());
        inputElement.className = this.options.className;
        this.inputElements.push(inputElement);
        this.bindChange();
        return inputElement;
    }
    onGui() {
        this.createContainerElement();
        const label = document.createElement('p');
        if (this.options.label)
            label.innerText = this.options.label;
        if (this.options.label && this.containerElement)
            (0, utils_1.mountElement)(label, this.containerElement);
        label.setAttribute(constants_1.ID_ATTRIBUTE, this.getId() + constants_1.LABEL_DEFINITION);
        label.className = constants_1.LABEL_CLASS_DEFAULT;
        const container = document.createElement(constants_1.DIV_ELEMENT);
        container.className = constants_1.FIELD_CONTAINER_CLASS_DEFAULT;
        this.options.schema.forEach((field) => {
            const input = this.createRadioInputElement(field.id, field.value);
            this.createRadioLabelElement(field.label, field.id, input, container);
        });
        if (this.containerElement && container)
            (0, utils_1.mountElement)(container, this.containerElement);
        this.createValidationElement();
        if (this.containerElement && this.validationElement)
            (0, utils_1.mountElement)(this.validationElement, this.containerElement);
    }
    handleValidatedField() {
        if (!this.getValidity()) {
            this.inputElements.forEach((el) => {
                var _a;
                el === null || el === void 0 ? void 0 : el.classList.add(constants_1.FORM_ERROR_CLASS_DEFAULT);
                el === null || el === void 0 ? void 0 : el.setAttribute(constants_1.ARIA_INVALID_ATTRIBUTE, 'true');
                el === null || el === void 0 ? void 0 : el.setAttribute(constants_1.ARIA_DESCRIBEDBY_ATTRIBUTE, ((_a = this.validationElement) === null || _a === void 0 ? void 0 : _a.getAttribute('id')) || '');
            });
            const vMessage = this.getValidationMessage();
            if (this.validationElement && vMessage) {
                this.validationElement.innerText = vMessage;
                this.validationElement.style.display = 'block';
            }
        }
        else {
            if (this.validationElement) {
                this.validationElement.style.display = 'none';
            }
            this.inputElements.forEach((el) => {
                el === null || el === void 0 ? void 0 : el.classList.remove(constants_1.FORM_ERROR_CLASS_DEFAULT);
                el === null || el === void 0 ? void 0 : el.removeAttribute(constants_1.ARIA_INVALID_ATTRIBUTE);
                el === null || el === void 0 ? void 0 : el.removeAttribute(constants_1.ARIA_DESCRIBEDBY_ATTRIBUTE);
            });
        }
        const form = this.getForm();
        form.updateError(this.getId(), this.getValidity());
    }
    bindChange() {
        if (this.inputElements[this.inputElements.length - 1])
            this.inputElements[this.inputElements.length - 1].addEventListener(constants_1.CHANGE_ATTRIBUTE, (event) => {
                this.change(event);
            });
    }
    getValue() {
        return this._value;
    }
}
exports.RadioField = RadioField;
