"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RangeField = void 0;
const constants_1 = require("../constants");
const field_1 = require("../field");
const utils_1 = require("../utils");
class RangeField extends field_1.Field {
    constructor(parent, form, options) {
        super(parent, form, options);
        this.options = {
            id: '',
            type: constants_1.FIELD_TYPE_RANGE,
            required: false,
            validation: (value, data, required) => {
                if (required && !value && value !== 0)
                    return constants_1.DEFAULT_REQUIRED_VALIDATION_MESSAGE;
                if (this.options.min && this.options.min > value)
                    return constants_1.BIGGER_THAN_VALIDATION_MESSAGE + this.options.min;
                if (this.options.max && this.options.max < value)
                    return constants_1.LESS_THAN_VALIDATION_MESSAGE + this.options.max;
                return true;
            },
            default: 0,
            className: constants_1.RANGE_CLASS_DEFAULT,
        };
        this.minElement = null;
        this.maxElement = null;
        this.valueElement = null;
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    createInputElement() {
        this.inputElement = document.createElement(constants_1.INPUT_ELEMENT);
        this.inputElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId());
        this.inputElement.setAttribute(constants_1.NAME_ATTRIBUTE, this.options.name || this.getId());
        this.inputElement.setAttribute(constants_1.TYPE_ATTRIBUTE, this.getType());
        if (this.options.min)
            this.inputElement.setAttribute(constants_1.MIN_ATTRIBUTE, String(this.options.min));
        if (this.options.max)
            this.inputElement.setAttribute(constants_1.MAX_ATTRIBUTE, String(this.options.max));
        if (this.options.step)
            this.inputElement.setAttribute(constants_1.STEP_ATTRIBUTE, String(this.options.step));
        this.inputElement.className = this.options.className;
    }
    createOtherElements() {
        this.minElement = document.createElement(constants_1.SPAN_ELEMENT);
        this.minElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId() + constants_1.MIN_DEFINITION);
        this.minElement.className = constants_1.RANGE_MIN_CLASS_DEFAULT;
        this.maxElement = document.createElement(constants_1.SPAN_ELEMENT);
        this.maxElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId() + constants_1.MAX_DEFINITION);
        this.maxElement.className = constants_1.RANGE_MAX_CLASS_DEFAULT;
        this.valueElement = document.createElement(constants_1.OUTPUT_ELEMENT);
        this.valueElement.setAttribute(constants_1.ID_ATTRIBUTE, this.getId() + constants_1.VALUE_DEFINITION);
        this.valueElement.className = constants_1.RANGE_VALUE_CLASS_DEFAULT;
    }
    onGui() {
        this.createContainerElement();
        this.createInputElement();
        this.createLabelElement();
        this.createValidationElement();
        this.createOtherElements();
        if (this.inputElement && this.labelElement)
            (0, utils_1.mountElement)(this.inputElement, this.labelElement);
        if (this.labelElement && this.validationElement)
            (0, utils_1.mountElement)(this.validationElement, this.labelElement);
        if (this.labelElement && this.valueElement)
            (0, utils_1.mountElement)(this.valueElement, this.labelElement);
        if (this.containerElement && this.labelElement)
            (0, utils_1.mountElement)(this.labelElement, this.containerElement);
    }
    updateRangeValues() {
        if (this.minElement && this.options.min !== undefined)
            this.minElement.innerText = String(this.options.min);
        if (this.maxElement && this.options.max !== undefined)
            this.maxElement.innerText = String(this.options.max);
        if (this.valueElement)
            this.valueElement.innerText = this.getValue() ? String(this.getValue()) : '';
        setTimeout(this.updateInputPadding, 10);
    }
    updateInputPadding() {
        if (this.inputElement && this.valueElement)
            this.inputElement.style.paddingRight = this.valueElement.clientWidth + 5 + 'px';
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.load();
            this.update();
            this.bindChange();
            this.updateRangeValues();
        });
    }
    reset() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            localStorage.removeItem(this.getSaveKey());
            this.setValue((_a = this.options.default) !== null && _a !== void 0 ? _a : null, false);
            this.updateRangeValues();
            this.update();
        });
    }
    bindChange() {
        if (this.inputElement) {
            this.inputElement.addEventListener(constants_1.CHANGE_ATTRIBUTE, (event) => {
                this.change(event);
                this.updateRangeValues();
            });
        }
    }
    getValue() {
        return this._value;
    }
}
exports.RangeField = RangeField;
