"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StaticField = void 0;
const utils_1 = require("../utils");
const constants_1 = require("../constants");
class StaticField {
    constructor(parent, form, options) {
        this.options = {
            id: '',
            type: constants_1.FIELD_TYPE_STATIC,
            template: '',
            className: constants_1.STATIC_CLASS_DEFAULT,
        };
        this.staticElement = null;
        this.containerElement = null;
        this._isMounted = false;
        this._isVisible = true;
        this._value = null;
        this._parsedConditions = null;
        this.initializeOptions(options);
        this._id = this.options.id;
        this._form = form;
        this._parent = parent;
        this._type = options.type;
        this._saveKey = (0, utils_1.generateFieldSaveKey)(this._form.getId(), this._id);
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, (0, utils_1.getOverwritenDefaults)(this.options.type, options));
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.parseStringConditions();
            this.load();
            this.update();
        });
    }
    parseStringConditions() {
        if (typeof this.options.conditions === 'string') {
            this._parsedConditions = (0, utils_1.parseConditionString)(this.options.conditions);
        }
    }
    getValue() {
        return this._value;
    }
    getId() {
        return this._id;
    }
    getSaveKey() {
        return this._saveKey;
    }
    getType() {
        return this._type;
    }
    getVisibility() {
        return this._isVisible;
    }
    getForm() {
        return this._form;
    }
    setTemplate(template, save = true) {
        this._value = template;
        if (this.staticElement)
            this.staticElement.innerHTML = template;
        if (save)
            this.save();
    }
    createContainerElement() {
        this.containerElement = document.createElement(constants_1.DIV_ELEMENT);
        this.containerElement.className = constants_1.FIELD_CLASS_DEFAULT + ' ' + this._type;
        this.containerElement.setAttribute(constants_1.ID_ATTRIBUTE, this._id + constants_1.CONTAINER_DEFINITION);
    }
    createStaticElement() {
        this.staticElement = document.createElement(constants_1.DIV_ELEMENT);
        this.staticElement.className = this.options.className;
        this.staticElement.setAttribute(constants_1.ID_ATTRIBUTE, this._id);
    }
    onGui() {
        this.createContainerElement();
        this.createStaticElement();
        if (this.containerElement && this.staticElement)
            (0, utils_1.mountElement)(this.staticElement, this.containerElement);
    }
    mount() {
        if (this.containerElement)
            (0, utils_1.mountElement)(this.containerElement, this._parent);
        this._isMounted = true;
    }
    unmount() {
        if (this.containerElement)
            (0, utils_1.unmountElement)(this.containerElement);
        this._isMounted = false;
    }
    destroy() {
        if (this._parent)
            this._parent.remove();
    }
    handleVisibility() {
        if (this._isVisible && !this._isMounted)
            this.mount();
        if (!this._isVisible && this._isMounted)
            this.unmount();
    }
    save() {
        if (this._form.savesProgress() && this._form.hasValidLicense() && this._value !== undefined) {
            localStorage.setItem(this._saveKey, JSON.stringify(this._value));
        }
    }
    load() {
        if (this._form.savesProgress() && this._form.hasValidLicense()) {
            const value = localStorage.getItem(this._saveKey);
            if (value !== null) {
                this.setTemplate((0, utils_1.isJson)(value) ? JSON.parse(value) : value, false);
                return;
            }
        }
        this.setTemplate(this.options.template, false);
    }
    reset() {
        return __awaiter(this, void 0, void 0, function* () {
            localStorage.removeItem(this._saveKey);
            this.setTemplate(this.options.template, false);
            this.update();
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            this.updateVisibilityBasedOnConditions();
            this.handleVisibility();
        });
    }
    updateVisibilityBasedOnConditions() {
        if (this.options.conditions) {
            if (this._parsedConditions) {
                this._isVisible = (0, utils_1.evaluateParsedConditions)(this._parsedConditions, this._form.getData());
            }
            else if (typeof this.options.conditions === 'function') {
                this._isVisible = this.options.conditions(this.getValue(), this._form.getData());
            }
        }
    }
}
exports.StaticField = StaticField;
