"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Group = void 0;
const utils_1 = require("./utils");
const constants_1 = require("./constants");
class Group {
    constructor(parent, form, options) {
        this.options = {
            id: '',
            type: constants_1.GROUP_TYPE_GROUP,
            className: constants_1.GROUP_CLASS_DEFAULT,
            prefixSchema: false,
            schema: [],
        };
        this.containerElement = null;
        this.schemaContainerElement = null;
        this.labelElement = null;
        this._isMounted = false;
        this._isVisible = true;
        this._parsedConditions = null;
        this.initializeOptions(options);
        this._parent = parent;
        this._form = form;
        this._id = this.options.id;
        this._type = options.type;
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, (0, utils_1.getOverwritenDefaults)(this.options.type, options));
    }
    initialize() {
        return __awaiter(this, void 0, void 0, function* () {
            this.parseStringConditions();
            yield this.reset();
        });
    }
    parseStringConditions() {
        if (typeof this.options.conditions === 'string') {
            this._parsedConditions = (0, utils_1.parseConditionString)(this.options.conditions);
        }
    }
    getId() {
        return this._id;
    }
    getContainer() {
        return this.containerElement;
    }
    getSchemaContainer() {
        return this.schemaContainerElement;
    }
    getType() {
        return this._type;
    }
    getVisibility() {
        return this._isVisible;
    }
    createContainerElement() {
        this.containerElement = document.createElement(constants_1.DIV_ELEMENT);
        this.containerElement.className = constants_1.GROUP_CONTAINER_CLASS_DEFAULT;
        this.containerElement.setAttribute(constants_1.ID_ATTRIBUTE, this._id + constants_1.GROUP_CONTAINER_DEFINITION);
    }
    createSchemaContainerElement() {
        this.schemaContainerElement = document.createElement(constants_1.DIV_ELEMENT);
        this.schemaContainerElement.className = this.options.className;
        this.schemaContainerElement.setAttribute(constants_1.ID_ATTRIBUTE, this._id);
    }
    createLabelElement() {
        this.labelElement = document.createElement(constants_1.H3_ELEMENT);
        if (this.options.label)
            this.labelElement.innerText = this.options.label;
        this.labelElement.setAttribute(constants_1.ID_ATTRIBUTE, this._id + constants_1.LABEL_DEFINITION);
        this.labelElement.className = constants_1.GROUP_LABEL_CLASS_DEFAULT;
    }
    onGui() {
        this.createContainerElement();
        this.createSchemaContainerElement();
        this.createLabelElement();
        if (this.containerElement && this.labelElement && this.options.label)
            (0, utils_1.mountElement)(this.labelElement, this.containerElement);
        if (this.containerElement && this.schemaContainerElement)
            (0, utils_1.mountElement)(this.schemaContainerElement, this.containerElement);
    }
    mount() {
        if (this.containerElement)
            (0, utils_1.mountElement)(this.containerElement, this._parent);
        this._isMounted = true;
    }
    unmount() {
        if (this.containerElement)
            (0, utils_1.unmountElement)(this.containerElement);
        this._isMounted = false;
    }
    destroy() {
        if (this._parent)
            this._parent.remove();
    }
    handleVisibility() {
        if (this._isVisible && !this._isMounted)
            this.mount();
        if (!this._isVisible && this._isMounted)
            this.unmount();
    }
    reset() {
        return __awaiter(this, void 0, void 0, function* () {
            this.update();
        });
    }
    update() {
        return __awaiter(this, void 0, void 0, function* () {
            this.updateVisibilityBasedOnConditions();
            this.handleVisibility();
        });
    }
    updateVisibilityBasedOnConditions() {
        if (this.options.conditions) {
            if (this._parsedConditions) {
                this._isVisible = (0, utils_1.evaluateParsedConditions)(this._parsedConditions, this._form.getData());
            }
            else if (typeof this.options.conditions === 'function') {
                this._isVisible = this.options.conditions(this._form.getData());
            }
        }
    }
}
exports.Group = Group;
