"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOverwritenDefaults = exports.overwriteDefaults = exports.isJson = exports.compareValues = exports.evaluateParsedConditions = exports.parseConditionString = exports.useLicensedFetures = exports.usesLicensedFetures = exports.handleInvalidLicenseLog = exports.getLicenseText = exports.setLicenseKey = exports.processLicenseKey = exports.generateFieldSaveKey = exports.transformFieldName = exports.createWrapper = exports.extractFieldsFromSchema = exports.getFormElementType = exports.isButton = exports.isGroup = exports.isField = exports.objectToFormData = exports.unmountElement = exports.mountElement = exports.debounce = void 0;
const constants_1 = require("./constants");
let LICENSE_KEY = null;
let USES_LICENSED_FETURES = false;
const OVEWRITE_DEFAULT_OPTIONS = {};
const debounce = (callback, wait, context) => {
    let timeoutId = null;
    return (...args) => {
        if (timeoutId)
            window.clearTimeout(timeoutId);
        timeoutId = window.setTimeout(() => {
            callback.apply(context, args);
        }, wait);
    };
};
exports.debounce = debounce;
const mountElement = (element, parent) => {
    parent.append(element);
};
exports.mountElement = mountElement;
const unmountElement = (element) => {
    var _a;
    (_a = element.parentNode) === null || _a === void 0 ? void 0 : _a.removeChild(element);
};
exports.unmountElement = unmountElement;
const objectToFormData = (obj, form, namespace) => {
    const fd = form || new FormData();
    let formKey;
    for (const property in obj) {
        if (Object.prototype.hasOwnProperty.call(obj, property)) {
            if (namespace) {
                formKey = namespace + '[' + property + ']';
            }
            else {
                formKey = property;
            }
            if (obj[property] == null) {
                fd.append(formKey, '');
            }
            else if (typeof obj[property] === 'object' && !(obj[property] instanceof File)) {
                (0, exports.objectToFormData)(obj[property], fd, property);
            }
            else {
                fd.append(formKey, obj[property]);
            }
        }
    }
    return fd;
};
exports.objectToFormData = objectToFormData;
const isField = (type) => {
    return constants_1.fields.includes(type);
};
exports.isField = isField;
const isGroup = (type) => {
    return constants_1.groups.includes(type);
};
exports.isGroup = isGroup;
const isButton = (type) => {
    return constants_1.buttons.includes(type);
};
exports.isButton = isButton;
const getFormElementType = (type) => {
    if ((0, exports.isField)(type))
        return constants_1.constructorTypes.field;
    if ((0, exports.isGroup)(type))
        return constants_1.constructorTypes.group;
    if ((0, exports.isButton)(type))
        return constants_1.constructorTypes.button;
    return null;
};
exports.getFormElementType = getFormElementType;
const extractFieldsFromSchema = (schema) => {
    let fieldsList = [];
    schema.forEach((options) => {
        if (constants_1.fields.includes(options.type))
            fieldsList.push(options.id);
        else if (options.schema)
            fieldsList = [...fieldsList, ...(0, exports.extractFieldsFromSchema)(options.schema)];
    });
    return fieldsList;
};
exports.extractFieldsFromSchema = extractFieldsFromSchema;
const createWrapper = (parent) => {
    const wrapper = document.createElement(constants_1.DIV_ELEMENT);
    (0, exports.mountElement)(wrapper, parent);
    return wrapper;
};
exports.createWrapper = createWrapper;
const transformFieldName = (parentId, key, id) => {
    return parentId + '[' + key + ']' + id;
};
exports.transformFieldName = transformFieldName;
const generateFieldSaveKey = (formId, id) => {
    return constants_1.FORMSJS_KEY_DEFINITION + formId + constants_1.FIELD_KEY_DEFINITION + id;
};
exports.generateFieldSaveKey = generateFieldSaveKey;
const isDateValid = (d) => d instanceof Date && !isNaN(d);
const calculateDateAfterAddingDays = (date, days) => {
    const result = new Date(date);
    result.setDate(result.getDate() + days);
    return result;
};
const parseKey = (key) => key.match(/^(\d+)@(\d+)$/);
const isKeyInLicenseKeys = (key) => constants_1.OS_LICENSE_KEYS.includes(key);
const calculateMinPurchaseDate = (releaseDate) => calculateDateAfterAddingDays(releaseDate, -constants_1.UPGRADE_WINDOW);
const determineLicenseState = (purchaseDate, minPurchaseDate) => minPurchaseDate < purchaseDate ? constants_1.LICENSE_STATE.VALID : constants_1.LICENSE_STATE.OUTDATED;
const processLicenseKey = () => {
    var _a;
    const key = LICENSE_KEY;
    if (!key)
        return constants_1.LICENSE_STATE.INVALID;
    if (isKeyInLicenseKeys(key))
        return constants_1.LICENSE_STATE.VALID;
    const parts = parseKey(key);
    if (((_a = parts === null || parts === void 0 ? void 0 : parts[1]) === null || _a === void 0 ? void 0 : _a.length) === 10) {
        const [, , timestamp] = parts;
        const purchaseDate = new Date(parseInt(timestamp, 10) * 1000);
        const releaseDate = new Date(constants_1.RELEASE_DATE);
        if (isDateValid(releaseDate)) {
            const minPurchaseDate = calculateMinPurchaseDate(releaseDate);
            return determineLicenseState(purchaseDate, minPurchaseDate);
        }
    }
    return constants_1.LICENSE_STATE.INVALID;
};
exports.processLicenseKey = processLicenseKey;
const setLicenseKey = (key) => {
    LICENSE_KEY = key;
};
exports.setLicenseKey = setLicenseKey;
const getLicenseText = (license) => {
    if (license === constants_1.LICENSE_STATE.VALID)
        return constants_1.VALID_LICENSE_TEXT;
    if (license === constants_1.LICENSE_STATE.OUTDATED)
        return constants_1.OUTDATED_LICENSE_TEXT;
    return constants_1.INVALID_LICENSE_TEXT;
};
exports.getLicenseText = getLicenseText;
const handleInvalidLicenseLog = (license) => {
    if (license === constants_1.LICENSE_STATE.INVALID) {
        console.error(constants_1.INVALID_CONSOLE_TEXT);
    }
    else {
        console.error(constants_1.OUTDATED_CONSOLE_TEXT);
    }
};
exports.handleInvalidLicenseLog = handleInvalidLicenseLog;
const usesLicensedFetures = () => {
    return USES_LICENSED_FETURES;
};
exports.usesLicensedFetures = usesLicensedFetures;
const useLicensedFetures = () => {
    USES_LICENSED_FETURES = true;
};
exports.useLicensedFetures = useLicensedFetures;
const parseConditionString = (conditionStr) => {
    return conditionStr
        .split(constants_1.STRING_CONDITION_SPLIT)
        .filter((part) => part.includes(constants_1.STRING_CONDITION_VALUE_SPLIT))
        .map((part) => {
        const [rawConditions, returnValue] = part.split(constants_1.STRING_CONDITION_VALUE_SPLIT);
        const conditions = rawConditions
            .replace(/^\[|\]$/g, '')
            .split('&&')
            .map((andCond) => andCond.split('||').map((orCond) => {
            const match = orCond.match(/(.*?)(=|!=|>|<|>=|<=)(.*)/);
            if (!match)
                throw new Error(`Invalid condition format: ${orCond}`);
            const [, left, operator, rightPart] = match;
            let right;
            const isDate = rightPart.trim().startsWith(constants_1.STRING_CONDITION_DATE_IDENTIFIER);
            if (isDate) {
                right = new Date(rightPart.trim().substring(5));
            }
            else {
                try {
                    right = JSON.parse(rightPart.trim());
                }
                catch (_a) {
                    right = rightPart.trim();
                }
            }
            return { left: left.trim(), operator: operator, right, isDate };
        }));
        return { conditions, returnValue };
    });
};
exports.parseConditionString = parseConditionString;
const evaluateParsedConditions = (parsedConditions, data, value = null, required = null, def = false) => {
    for (const { conditions, returnValue } of parsedConditions) {
        const conditionResult = conditions.every((andConditions) => andConditions.some(({ left, operator, right }) => {
            const dataList = Object.assign({ _value: value, _required: required }, data);
            const leftValue = getNestedValue(dataList, left);
            return (0, exports.compareValues)(operator, leftValue, right);
        }));
        if (conditionResult) {
            try {
                return JSON.parse(returnValue);
            }
            catch (_a) {
                return returnValue;
            }
        }
    }
    return def;
};
exports.evaluateParsedConditions = evaluateParsedConditions;
const getNestedValue = (data, path) => {
    return path.split('.').reduce((acc, part) => acc && acc[part], data);
};
const compareValues = (operator, a, b) => {
    switch (operator) {
        case '=':
            return a === b;
        case '!=':
            return a !== b;
        case '>':
            return a > b;
        case '<':
            return a < b;
        case '>=':
            return a >= b;
        case '<=':
            return a <= b;
    }
};
exports.compareValues = compareValues;
const isJson = (str) => {
    try {
        JSON.parse(str);
    }
    catch (e) {
        return false;
    }
    return true;
};
exports.isJson = isJson;
const overwriteDefaults = (type, defaults) => {
    OVEWRITE_DEFAULT_OPTIONS[type] = defaults;
};
exports.overwriteDefaults = overwriteDefaults;
const getOverwritenDefaults = (type, defaults) => {
    if (Object.keys(OVEWRITE_DEFAULT_OPTIONS).includes(type)) {
        return Object.assign({}, defaults, OVEWRITE_DEFAULT_OPTIONS[type]);
    }
    return defaults;
};
exports.getOverwritenDefaults = getOverwritenDefaults;
