import { CHANGE_ATTRIBUTE, CHECKBOX_CLASS_DEFAULT, CHECKED_ATTRIBUTE, CONTAINER_DEFINITION, DEFAULT_REQUIRED_VALIDATION_MESSAGE, DIV_ELEMENT, FIELD_CLASS_DEFAULT, FIELD_TYPE_CHECKBOX, FOR_ATTRIBUTE, ID_ATTRIBUTE, INPUT_ELEMENT, LABEL_CLASS_DEFAULT, LABEL_DEFINITION, LABEL_ELEMENT, NAME_ATTRIBUTE, PARAGRAPH_ELEMENT, SPAN_ELEMENT, TOGGLE_CLASS_DEFAULT, TOGGLE_SLIDER_CLASS_DEFAULT, TYPE_ATTRIBUTE, } from '../constants';
import { Field } from '../field';
import { mountElement } from '../utils';
export class CheckboxField extends Field {
    options = {
        id: '',
        type: FIELD_TYPE_CHECKBOX,
        required: false,
        validation: (value, data, required) => {
            if (required && !value)
                return DEFAULT_REQUIRED_VALIDATION_MESSAGE;
            return true;
        },
        default: null,
        className: CHECKBOX_CLASS_DEFAULT,
        toggle: false,
    };
    sliderElement = null;
    constructor(parent, form, options) {
        super(parent, form, options);
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    syncValue() {
        if (this.inputElement && this.inputElement instanceof HTMLInputElement) {
            if (this.getValue() && !this.inputElement.hasAttribute(CHECKED_ATTRIBUTE)) {
                this.inputElement.setAttribute(CHECKED_ATTRIBUTE, String(this.getValue()));
            }
            else if (!this.getValue() && this.inputElement.hasAttribute(CHECKED_ATTRIBUTE))
                this.inputElement.removeAttribute(CHECKED_ATTRIBUTE);
        }
    }
    createContainerElement() {
        this.containerElement = document.createElement(DIV_ELEMENT);
        this.containerElement.className = FIELD_CLASS_DEFAULT + ' ' + this.getType();
        if (this.options.toggle)
            this.containerElement.className += ' ' + TOGGLE_CLASS_DEFAULT;
        this.containerElement.setAttribute(ID_ATTRIBUTE, this.getId() + CONTAINER_DEFINITION);
    }
    createLabelElement() {
        this.labelElement = document.createElement(LABEL_ELEMENT);
        if (this.inputElement && this.labelElement)
            mountElement(this.inputElement, this.labelElement);
        if (this.sliderElement && this.labelElement)
            mountElement(this.sliderElement, this.labelElement);
        const label = document.createElement(PARAGRAPH_ELEMENT);
        if (this.options.label)
            label.innerText = this.options.label;
        label.setAttribute(ID_ATTRIBUTE, this.getId() + LABEL_DEFINITION);
        this.labelElement.setAttribute(FOR_ATTRIBUTE, this.options.name || this.getId());
        label.className = LABEL_CLASS_DEFAULT;
        if (this.options.label)
            mountElement(label, this.labelElement);
    }
    createInputElement() {
        this.inputElement = document.createElement(INPUT_ELEMENT);
        this.inputElement.setAttribute(ID_ATTRIBUTE, this.getId());
        this.inputElement.setAttribute(NAME_ATTRIBUTE, this.options.name || this.getId());
        this.inputElement.setAttribute(TYPE_ATTRIBUTE, this.getType());
        if (this.options.default)
            this.inputElement.setAttribute(CHECKED_ATTRIBUTE, String(this.options.default));
        this.inputElement.className = this.options.className;
    }
    createToggleSpan() {
        this.sliderElement = document.createElement(SPAN_ELEMENT);
        this.sliderElement.className = TOGGLE_SLIDER_CLASS_DEFAULT;
    }
    onGui() {
        this.createContainerElement();
        if (this.options.toggle)
            this.createToggleSpan();
        this.createInputElement();
        this.createLabelElement();
        this.createValidationElement();
        if (this.containerElement && this.labelElement)
            mountElement(this.labelElement, this.containerElement);
        if (this.labelElement && this.validationElement)
            mountElement(this.validationElement, this.labelElement);
    }
    bindChange() {
        if (this.inputElement)
            this.inputElement.addEventListener(CHANGE_ATTRIBUTE, (event) => {
                this.change(event);
            });
    }
    change(event) {
        this.setValue(event.target.checked);
        if (this.options.change)
            this.options.change(this.getValue());
        this.validate();
    }
    getValue() {
        return this._value;
    }
}
