import * as flatpickerNamespace from 'flatpickr';
const flatpickr = flatpickerNamespace.default;
import { Field } from '../field';
import { CHANGE_ATTRIBUTE, DEFAULT_REQUIRED_VALIDATION_MESSAGE, FIELD_TYPE_DATETIME, ID_ATTRIBUTE, INPUT_CLASS_DEFAULT, INPUT_ELEMENT, NAME_ATTRIBUTE, PLACEHOLDER_ATTRIBUTE, TYPE_ATTRIBUTE, } from '../constants';
export class DatetimeField extends Field {
    options = {
        id: '',
        type: FIELD_TYPE_DATETIME,
        required: false,
        validation: (value, data, required) => {
            if (required && !value)
                return DEFAULT_REQUIRED_VALIDATION_MESSAGE;
            return true;
        },
        className: INPUT_CLASS_DEFAULT,
        enhance: true,
        options: {
            altInput: true,
            enableTime: true,
        },
    };
    _flatpickr = null;
    constructor(parent, form, options) {
        super(parent, form, options);
        this.initializeOptions(options);
        this.onGui();
        this.initialize();
    }
    async initialize() {
        this.load();
        this.update();
        this.initFlatpickr();
        this.bindChange();
    }
    getFlatpickr() {
        return this._flatpickr;
    }
    initFlatpickr() {
        if (this.inputElement && this.options.enhance)
            this._flatpickr = flatpickr(this.inputElement, this.options.options || {});
    }
    bindChange() {
        if (this.inputElement)
            this.inputElement.addEventListener(CHANGE_ATTRIBUTE, (event) => {
                this.change(event);
            });
    }
    createInputElement() {
        this.inputElement = document.createElement(INPUT_ELEMENT);
        this.inputElement.setAttribute(ID_ATTRIBUTE, this.getId());
        this.inputElement.setAttribute(NAME_ATTRIBUTE, this.options.name || this.getId());
        this.inputElement.setAttribute(TYPE_ATTRIBUTE, this.options.enhance ? 'text' : 'datetime-local');
        this.inputElement.className = this.options.className;
        if (this.options.placeholder)
            this.inputElement.setAttribute(PLACEHOLDER_ATTRIBUTE, this.options.placeholder);
    }
    getValue() {
        return this._value;
    }
}
