import { evaluateParsedConditions, getOverwritenDefaults, mountElement, parseConditionString, unmountElement, } from './utils';
import { GROUP_TYPE_GROUP, DIV_ELEMENT, GROUP_CLASS_DEFAULT, GROUP_CONTAINER_CLASS_DEFAULT, GROUP_CONTAINER_DEFINITION, GROUP_LABEL_CLASS_DEFAULT, H3_ELEMENT, ID_ATTRIBUTE, LABEL_DEFINITION, } from './constants';
export class Group {
    options = {
        id: '',
        type: GROUP_TYPE_GROUP,
        className: GROUP_CLASS_DEFAULT,
        prefixSchema: false,
        schema: [],
    };
    containerElement = null;
    schemaContainerElement = null;
    labelElement = null;
    _id;
    _parent;
    _form;
    _isMounted = false;
    _isVisible = true;
    _type;
    _parsedConditions = null;
    constructor(parent, form, options) {
        this.initializeOptions(options);
        this._parent = parent;
        this._form = form;
        this._id = this.options.id;
        this._type = options.type;
        this.onGui();
        this.initialize();
    }
    initializeOptions(options) {
        this.options = Object.assign({}, this.options, getOverwritenDefaults(this.options.type, options));
    }
    async initialize() {
        this.parseStringConditions();
        await this.reset();
    }
    parseStringConditions() {
        if (typeof this.options.conditions === 'string') {
            this._parsedConditions = parseConditionString(this.options.conditions);
        }
    }
    getId() {
        return this._id;
    }
    getContainer() {
        return this.containerElement;
    }
    getSchemaContainer() {
        return this.schemaContainerElement;
    }
    getType() {
        return this._type;
    }
    getVisibility() {
        return this._isVisible;
    }
    createContainerElement() {
        this.containerElement = document.createElement(DIV_ELEMENT);
        this.containerElement.className = GROUP_CONTAINER_CLASS_DEFAULT;
        this.containerElement.setAttribute(ID_ATTRIBUTE, this._id + GROUP_CONTAINER_DEFINITION);
    }
    createSchemaContainerElement() {
        this.schemaContainerElement = document.createElement(DIV_ELEMENT);
        this.schemaContainerElement.className = this.options.className;
        this.schemaContainerElement.setAttribute(ID_ATTRIBUTE, this._id);
    }
    createLabelElement() {
        this.labelElement = document.createElement(H3_ELEMENT);
        if (this.options.label)
            this.labelElement.innerText = this.options.label;
        this.labelElement.setAttribute(ID_ATTRIBUTE, this._id + LABEL_DEFINITION);
        this.labelElement.className = GROUP_LABEL_CLASS_DEFAULT;
    }
    onGui() {
        this.createContainerElement();
        this.createSchemaContainerElement();
        this.createLabelElement();
        if (this.containerElement && this.labelElement && this.options.label)
            mountElement(this.labelElement, this.containerElement);
        if (this.containerElement && this.schemaContainerElement)
            mountElement(this.schemaContainerElement, this.containerElement);
    }
    mount() {
        if (this.containerElement)
            mountElement(this.containerElement, this._parent);
        this._isMounted = true;
    }
    unmount() {
        if (this.containerElement)
            unmountElement(this.containerElement);
        this._isMounted = false;
    }
    destroy() {
        if (this._parent)
            this._parent.remove();
    }
    handleVisibility() {
        if (this._isVisible && !this._isMounted)
            this.mount();
        if (!this._isVisible && this._isMounted)
            this.unmount();
    }
    async reset() {
        this.update();
    }
    async update() {
        this.updateVisibilityBasedOnConditions();
        this.handleVisibility();
    }
    updateVisibilityBasedOnConditions() {
        if (this.options.conditions) {
            if (this._parsedConditions) {
                this._isVisible = evaluateParsedConditions(this._parsedConditions, this._form.getData());
            }
            else if (typeof this.options.conditions === 'function') {
                this._isVisible = this.options.conditions(this._form.getData());
            }
        }
    }
}
